@@ -209,8 +209,11 @@ struct msgb *iofd_msgb_pending_or_alloc(struct osmo_io_fd *iofd)
  */
 int iofd_txqueue_enqueue(struct osmo_io_fd *iofd, struct iofd_msghdr *msghdr)
 {
-	if (iofd->tx_queue.current_length >= iofd->tx_queue.max_length)
+	if (iofd->tx_queue.current_length >= iofd->tx_queue.max_length) {
+		LOGPIO(iofd, LOGL_ERROR, "enqueueing message failed (queue full, %u msgs). Rejecting msgb\n",
+		       iofd->tx_queue.current_length);
 		return -ENOSPC;
+	}
 
 	llist_add_tail(&msghdr->list, &iofd->tx_queue.msg_queue);
 	iofd->tx_queue.current_length++;
@@ -520,7 +523,6 @@ int osmo_iofd_sendto_msgb(struct osmo_io_fd *iofd, struct msgb *msg, int sendto_
 	rc = iofd_txqueue_enqueue(iofd, msghdr);
 	if (rc < 0) {
 		iofd_msghdr_free(msghdr);
-		LOGPIO(iofd, LOGL_ERROR, "enqueueing message failed (%d). Rejecting msgb\n", rc);
 		return rc;
 	}
 
@@ -596,7 +598,6 @@ int osmo_iofd_sendmsg_msgb(struct osmo_io_fd *iofd, struct msgb *msg, int sendms
 	rc = iofd_txqueue_enqueue(iofd, msghdr);
 	if (rc < 0) {
 		iofd_msghdr_free(msghdr);
-		LOGPIO(iofd, LOGL_ERROR, "enqueueing message failed (%d). Rejecting msgb\n", rc);
 		return rc;
 	}
 
@@ -745,21 +746,23 @@ int osmo_iofd_register(struct osmo_io_fd *iofd, int fd)
 {
 	int rc = 0;
 
-	if (fd < 0 && iofd->fd < 0) {
+	if (IOFD_FLAG_ISSET(iofd, IOFD_FLAG_FD_REGISTERED)) {
+		/* If re-registering same fd, handle as NO-OP.
+		 * And it is an even more explicit NO-OP
+		 * if the caller passed in -1. */
+		if (fd < 0 || fd == iofd->fd)
+			return 0;
+		/* New FD should go through unregister() first. */
+		return -ENOTSUP;
+	}
+
+	if (fd >= 0)
+		iofd->fd = fd;
+	if (iofd->fd < 0) {
 		/* this might happen if both osmo_iofd_setup() and osmo_iofd_register() are called with -1 */
 		LOGPIO(iofd, LOGL_ERROR, "Cannot register io_fd using invalid fd == %d\n", iofd->fd);
 		return -EBADF;
 	}
-	if (fd < 0)
-		fd = iofd->fd;
-	else if (iofd->fd < 0)
-		iofd->fd = fd;
-
-	if (IOFD_FLAG_ISSET(iofd, IOFD_FLAG_FD_REGISTERED)) {
-		/* If re-registering same fd, handle as NO-OP.
-		 * New FD should go through unregister() first. */
-		return iofd->fd == fd ? 0 : -ENOTSUP;
-	}
 
 	rc = osmo_iofd_ops.register_fd(iofd);
 	if (rc)
